#include "x24x_inc.h"      
#include "global.h"
#include <math.h>

#define NBRE_PAS 4000  

#define D_ROUE_DROIT 0.070145                /*Valeur 4eme match : 0.07015*/ /*0.070	                /* en m */ 
#define D_ROUE_GAUCHE 0.069855               /*Valeur 4eme match : 0.06985*/	/*0.070	                /* en m */
#define ECART_ROUE_LIBRE 0.0875  	        /* en m */  /*ancien : 11.8*/
#define ECART_ROUE_MOTRICE 0.230	        /* en m */
#define PERIMETRE_DROIT (PI*D_ROUE_DROIT)	/* primtre utilis pour les deux roues */
#define PERIMETRE_GAUCHE (PI*D_ROUE_GAUCHE) /* valeur non utilise */

#define TRUE 1
#define FALSE 0
#define DIST_MIN 0.04	/* en m */
#define V_MIN 0.1       /* en m/s */
#define KDIST 5000/*3000         /* gain proportionnel de position -> "acceleration"  donner au robot */   
#define KTETA 1000/*1800*/        /* gain proportionnel de position pour la rotation */    
#define KP_DROIT 3.0	/*3.0		/* gain proportionnel du rgulateur de vitesse droit*/
#define TI_DROIT 0.8	/*0.8		/* temps d'intgration 			"				 "	*/
#define KP_GAUCHE 3.0	/*3.0		/* gain proportionnel			" 			   gauche*/
#define TI_GAUCHE 0.8	/*0.8		/* temps d'intgration 			"				 " 	*/   

#define TE 0.01					/* priode d'chantillonnage */
#define TM 0.8					/* constante de temps des moteurs */
#define VMAX 900	/*0*/				/* vitesse max de consigne */
#define THETA0 (PI/6)
#define C (VMAX*TE)/(0.8*TM)	/* pente de l'acclration maximale de la consigne moyenne 
									|-> il faut 0.8*TM secondes pour passer de 0  VMAX*/
#define TEMPS_BLOCAGE 100		/* (TEMPS_BLOCAGE/100) secondes = temps maximum pendant lequel le robot peut tre bloqu */
#define DIST_TEST_BLOCAGE 0.002		/* on considre tre bloqu 
									si ecart entre (moyenne des TEMPS_BLOCAGE derniers de x et y)
									et (position actuel du robot)
									infrieur a DIST_TEST_BLOCAGE */
#define TETA_TEST_BLOCAGE ((5*PI)/180)	/* on considre tre bloqu 
										si ecart entre (moyenne des TEMPS_BLOCAGE derniers de teta)
										et (position actuel du robot)
										infrieur a DIST_TEST_BLOCAGE */
										

/* Systeme d'anti-blocage */
double x_position_backup[TEMPS_BLOCAGE];
double y_position_backup[TEMPS_BLOCAGE];
double teta_position_backup[TEMPS_BLOCAGE];

int anti_block_pos = 0;

/* memo compteur impulsions */
int memo_imp_droite= 0, memo_imp_gauche= 0;

/* coordonnes du robot */
double x_robot = 0.0, y_robot = 0.0, teta_robot = 0.0;
/* coordonnes du point d'arrive */
double x_arrivee, y_arrivee;

/* Vitesse maximum demande */
double vitesse_maximum;

int gain_rotation;

/* activateur du dplacement */
int ordre_deplacement = ASSERV_IDLE;

/* paramtres de l'asservissement */  
	/* donnes des rgulateurs PI */
double Integ_Er_droite, Integ_Er_gauche;
int delta_imp_droite, delta_imp_gauche;	/* avancement des roues libres en nombre de pas codeur */
double Vmes_droite, Vmes_gauche;	/* vitesse des roues motrices en m/s */          
double Vmoy = 0.0;						/* vitesse de consigne moyenne */

#define NBSECONDS 10
#define NBPOINTS (NBSECONDS*100)

int table_ErDist[NBPOINTS];
int table_ErTeta[NBPOINTS];
int table_ConsGauche[NBPOINTS];
int table_ConsDroite[NBPOINTS];

/* Position en X du robot */
int table[NBPOINTS];

/* Position en Y du robot */
int table2[NBPOINTS];

int tablereg[NBPOINTS];
int tablereg2[NBPOINTS];
int table_TetaRobotArrivee[NBPOINTS];
int table_TetaRobot[NBPOINTS];  
int table_VitGauche[NBPOINTS];
int table_VitDroite[NBPOINTS];
int table_TempsCalcul[NBPOINTS];
int i = 0;


double ervit1,ervit2;                         
int tablevit1[NBPOINTS],tablevit2[NBPOINTS];

int timer = -1;

void positionnement(void);
double Erreur_distance(void);
double Teta_RobotArrivee(void);
double Erreur_orientation(void);   
double Vitesse_moyenne(double distance, double Vmoyen, double ErreurTheta);                
double Consigne_droite(double vitesse, double angle);
double Consigne_gauche(double vitesse, double angle);
double Erreur_vitesse(double consigne, double mesure);
void Initialisation_asservissement(double coord_x_arrivee, double coord_y_arrivee, int bouge, int grotation);
double Regulateur_PI_droit(double Er);
double Regulateur_PI_gauche(double Er);


void c_int2 ()
{   
	double ErDist;		/* distance en ligne droite entre le robot et le point d'arrivee */                     
    double ErTeta;		/* angle entre la droite passant par le robot et le point d'arrive, et la direction du robot */			
    double Vcons_droite,Vcons_gauche;	/* consignes de vitesse  appliquer aux moteurs */
    int moyenneidx;
    double moyenne_x;
    double moyenne_y;
    double moyenne_teta; 
    
	
    asm("	CLRC INTM");
	
	/* mise des interruptions */
	IFR=IFR | 0x2;
	EVBIFRA=0xFFFF;          
	
	/* Le programme est en fonctionnement, on dcrmente le timer */	
	if(timer > 0)
	{
	  timer--;
	}
	/* On est arriv  expiration du temps */
	else if(timer == 0)
	{
	  /* Arret complet des moteurs */	  
	  CMPR1 = 1024;
	  CMPR2 = 1024;
	  
	  /* Dsactivation des interruptions */
	  asm("	SETC INTM");
	  
	  /* Arret du programme */
	  while(1);
	}
	
	
    /* actualisation de la pose du robot*/
	positionnement();
	
	
	if (ordre_deplacement == ASSERV_RUNNING)
	{
		ErDist = Erreur_distance();  			
		ErTeta = Erreur_orientation();                          

		/** Mesures de ErDist et de ErTeta ********/		                             
		if (i< NBPOINTS)
		{   
			if (i==0)
			{
				for(i=0;i<NBPOINTS;i=i+1) 
				{
					table_ErDist[i]=0;
					table_ErTeta[i]=0;
				}
				i=0;
			}              
			table_ErDist[i]=0;
			table_ErTeta[i]=0;
			table_ErDist[i]=(int)(100*ErDist);
			table_ErTeta[i]=(int)((180/PI)*ErTeta);
			
		}

        /* Si la commande tait de rester sur place, alors la condition 
           d'arret est sur l'erreur d'angle */		
		if((vitesse_maximum == 0) && (ErTeta < 0.1) && (ErTeta > -0.1))
		{
		    ordre_deplacement = ASSERV_IDLE;
		}
		
		/* Sinon, la commande est de se dplacer, alors la condition d'arret
		   est sur la distance */                             
		if ((ErDist < DIST_MIN) /*&& (Vmes_droite < V_MIN) && (Vmes_gauche < V_MIN)*/)
		/*if (ErTeta < 0.1)*/
		{
			ordre_deplacement = ASSERV_IDLE;
		}
	}
	
	if (ordre_deplacement == ASSERV_RUNNING)
	{
		Vmoy = Vitesse_moyenne(ErDist, Vmoy, ErTeta);
    		/* Moteur gauche */
		Vcons_gauche = Consigne_gauche(Vmoy, ErTeta);
		ervit1 = Erreur_vitesse(Vcons_gauche, Vmes_gauche*1765);
		CMPR1 = (int)Regulateur_PI_gauche(ervit1);	
										/* 1765 : permet de passer de m/s en valeur du registre PWM, 
										consigne tant entre -1024 et 1024,
										sachant qu' la valeur de vitesse max de 1024 dans le PWM, on a une vitesse de 0.58 m/s */		
		
		
		if (i< NBPOINTS)
		{   
			if (i==0)
			{
				for(i=0;i<NBPOINTS;i=i+1) 				{
					tablevit1[i]=0;
					table_VitGauche[i]=0;
				}
				i=0;
			}              
			tablevit1[i]=(int)(ervit1);	
			table_VitGauche[i] = Vmes_gauche*1765;
		}
		
		
			/* Moteur droit */
		Vcons_droite = Consigne_droite(Vmoy, ErTeta);
		ervit2 = Erreur_vitesse(Vcons_droite, Vmes_droite*1765);
		CMPR2 = (int)Regulateur_PI_droit(ervit2);
		
		
		if (i< NBPOINTS)
		{   
			if (i==0)
			{
				for(i=0;i<NBPOINTS;i=i+1) 
				{
					tablevit2[i]=0;
					table_VitDroite[i]=0;
				}
				i=0;
			}              
			tablevit2[i]=(int)(ervit2);
			table_VitDroite[i] = Vmes_droite*1765;	
		}
		
		
		
		/** Mesures de ConsDroitre et de ConsGauche ********/
		if (i< NBPOINTS)
		{   
			if (i==0)
			{
				for(i=0;i<NBPOINTS;i=i+1) 
				{
					table_ConsDroite[i]=0;
					table_ConsGauche[i]=0;
				}
				i=0;
			}              
			table_ConsDroite[i]=0;
			table_ConsGauche[i]=0;
			table_ConsDroite[i]=(int)(Vcons_droite);
			table_ConsGauche[i]=(int)(Vcons_gauche);			
		}
		/*
		/***************************************************/		 
	    
	    /* Systme anti blocage : on note les valeurs */
	    x_position_backup[anti_block_pos] = x_robot;
	    y_position_backup[anti_block_pos] = y_robot;
	    teta_position_backup[anti_block_pos] = teta_robot;
	    
	    anti_block_pos++;
	    if(anti_block_pos >= TEMPS_BLOCAGE)
	    {
	      anti_block_pos = 0;
	    }
	    
	    /* moyenne des coordonnees x, y et teta sur les (TEMPS_BLOCAGE/100) secondes */	    
	    moyenne_x = 0.0;
	    moyenne_y = 0.0;
	    moyenne_teta = 0.0;
	    for(moyenneidx = 0; moyenneidx < TEMPS_BLOCAGE; moyenneidx++)
	    {
	    	moyenne_x = moyenne_x + x_position_backup[moyenneidx];
	    	moyenne_y = moyenne_y + y_position_backup[moyenneidx];
	    	moyenne_teta = moyenne_teta + teta_position_backup[moyenneidx];
	    }
	    moyenne_x = moyenne_x / TEMPS_BLOCAGE;
	    moyenne_y = moyenne_y / TEMPS_BLOCAGE;
	    moyenne_teta = moyenne_teta / TEMPS_BLOCAGE;
	    
	    if ( ( ((moyenne_x - x_robot) < DIST_TEST_BLOCAGE) && ((moyenne_x - x_robot) > -DIST_TEST_BLOCAGE) )
			&& 	( ((moyenne_y - y_robot) < DIST_TEST_BLOCAGE) && ((moyenne_y - y_robot) > -DIST_TEST_BLOCAGE) )
			&&	( ((moyenne_teta - teta_robot) < TETA_TEST_BLOCAGE) && ((moyenne_teta - teta_robot) > -TETA_TEST_BLOCAGE) ) )
		{
			ordre_deplacement = ASSERV_BLOCK;	
		}
	}
	
	else
	{   /* si l'ordre de dplacement est faux, arrt complet du robot */
		CMPR1 = 1024;
		CMPR2 = 1024;
		
	}

	
	/* mesure du temps de calcul du programme d'asservissement */
	/*T3CON = T3CON & 0xE7FF;	/* le timer 3 est stopp */
	/*
	if (i< NBPOINTS)
		{   
			if (i==0)
			{
				for(i=0;i<NBPOINTS;i=i+1) 
				{
					table_TempsCalcul[i]=0;
				}
				i=0;
			}              
			table_TempsCalcul[i]=T3CNT;
		}
	
	T3CON = T3CON | 0x1000;	/* le timer 3 reprend */
	
	                                       
	/* incrmentation de l'index des tables de mesure */
	i++;
	
	
}

/* calcul de la nouvelle position du robot */ 
void positionnement(void)
{
	double delta_moy, delta_diff;
	double dx=0.0, dy=0.0;
	double dteta;		/* angle de rotation du robot entre deux calcul de d'asservissement */
	double vit_roue_libre_droite,vit_roue_libre_gauche;
	double rayon_courbure;
	int sens_virage;
  	
  	
  	/* calcul de l'avancement de chaque roue libre en pas codeur */
  		/* roue droite */
  	delta_imp_droite = T4CNT - memo_imp_droite;
  	memo_imp_droite = T4CNT;
  		/* roue gauche */ 
  	delta_imp_gauche = T2CNT - memo_imp_gauche;
  	memo_imp_gauche = T2CNT;
  	
  	
	delta_moy  = ((double)(delta_imp_droite*PERIMETRE_DROIT + delta_imp_gauche*PERIMETRE_GAUCHE)) / (2.0 * ((double)NBRE_PAS));
	delta_diff = ((double)(delta_imp_droite*PERIMETRE_DROIT - delta_imp_gauche*PERIMETRE_GAUCHE)) / ((double)NBRE_PAS);
    
    dx = delta_moy * cos(teta_robot);
    dy = delta_moy * sin(teta_robot);
    dteta = delta_diff/ ((double)ECART_ROUE_LIBRE);
    
    x_robot = x_robot + dx;
    y_robot = y_robot + dy;
    teta_robot = teta_robot + dteta;            
        
    if ( teta_robot > PI)
		{
			teta_robot = teta_robot - (2*PI);
		}
		else if (teta_robot  <= -PI)
		{
			teta_robot = teta_robot + (2*PI);
		}                            	
    
     /** Mesures de x_robot et de y_robot ********/		                             
    if (i< NBPOINTS)
		{   
			if (i==0)
			{
				for(i=0;i<NBPOINTS;i=i+1) table[i]=0; 
				for(i=0;i<NBPOINTS;i=i+1) table2[i]=0;
				for(i=0;i<NBPOINTS;i=i+1) table_TetaRobot[i]=0;
				i=0;
			}              
			table[i]=0; 
			table2[i]=0;
			table[i]=(int)(100*x_robot); 
			table2[i]=(int)(100*y_robot);
			table_TetaRobot[i]=(int)((180/PI)*teta_robot);
		}
	/*
	/*********************************************/	  
    
    
    /** calcul des vitesses des roues motrices **/

	/* roue droite */
	vit_roue_libre_droite = ((double)delta_imp_droite * ((double)PERIMETRE_DROIT)) / ((double)NBRE_PAS);   
   	vit_roue_libre_droite = vit_roue_libre_droite / TE;	/* unit = m/s */
              
	
    /* roue gauche */	
	vit_roue_libre_gauche = ((double)delta_imp_gauche * ((double)PERIMETRE_GAUCHE)) / ((double)NBRE_PAS);   
    vit_roue_libre_gauche = vit_roue_libre_gauche / TE;	/* unit = m/s */

    	/* test : le robot va tout droit */
    if (delta_diff == 0)/*(TRUE)*/
    {                       
    	Vmes_droite = vit_roue_libre_droite;
    	Vmes_gauche = vit_roue_libre_gauche;
    }
    else
    {   
    	rayon_courbure = delta_moy / dteta;
    	/* test virage  gauche */
    	if (delta_diff > 0)
    	{                  
    		sens_virage = 1;
		}
		/* test virage  droite */
		else
		{                   
			sens_virage = -1;		
		} 
			                                                                                                  
 		Vmes_droite = ( ((double)(sens_virage * ECART_ROUE_MOTRICE/2 + rayon_courbure)) * ((double)(1/(sens_virage * ECART_ROUE_LIBRE/2 + rayon_courbure))) ) * vit_roue_libre_droite;
			    
 		Vmes_gauche = ( ((double)(-sens_virage * ECART_ROUE_MOTRICE/2 + rayon_courbure)) * ((double)(1/(-sens_virage * ECART_ROUE_LIBRE/2 + rayon_courbure))) ) * vit_roue_libre_gauche;
                                                                                                                   
	}
}




/* distance en ligne droite entre le robot et le point d'arrivee */                     
double Erreur_distance(void)
	{
		return(sqrt((x_arrivee - x_robot)*(x_arrivee - x_robot) + (y_arrivee - y_robot )*(y_arrivee - y_robot) ));
	}
     
/* angle absolu de la droite passant par le robot et le point d'arrive */     
double Teta_RobotArrivee(void)
	{  
		/* Le teta  calculer pour s'orienter vers le point d'arrivee */ 
		double teta;                                                    
		
		/* La distance entre le robot et le point d'arrive */
		double distance;                                      
	
		/* Le cosinus de l'angle entre le robot et le point d'arrive */
		double cos_angle;
				                                                        
				                                                     
		/* Calcul de la distance robot -> arrive */
		distance = (sqrt((x_arrivee - x_robot)*(x_arrivee - x_robot) + (y_arrivee - y_robot )*(y_arrivee - y_robot) ));		       
		                                            
		
		cos_angle = (double)( x_arrivee - x_robot)/(double)(distance);
        
        /* On s'assure que le cosinus de l'angle est bien entre -1 et 1, sinon acos(< -1) = 0 */
		if(cos_angle < -1)
		{
			cos_angle = -1;			
		}
        else if(cos_angle > 1)
        {
          	cos_angle = 1;
        }             

		/* En fonction du cot du point d'arrive par rapport au robot, angle ngatif ou positif */		
		if((y_arrivee-y_robot)<0)
		{   
        	teta = -acos(cos_angle);
		}   
		else
		{
			teta = acos(cos_angle);
		}           

		/** enregistrement de l'angle ********/		                             
		if (i< NBPOINTS)
		{   
			if (i==0)
			{
				for(i=0;i<NBPOINTS;i=i+1) 
				{
					table_TetaRobotArrivee[i]=0;
				}
				i=0;
			}                      
			
			table_TetaRobotArrivee[i]=(int)((180/PI) * teta);
		}
		/********************************************/		                             
		
		return teta;				
	}
			
/* angle entre la droite passant par le robot et le point d'arrive, et la direction du robot */			
double Erreur_orientation(void)
	{    
		double temp; 
		
		temp = (Teta_RobotArrivee() - teta_robot);
		if (temp  > PI)
		{
			temp = temp - (2*PI);
		}
		else if (temp  < -PI)
		{
			temp = temp + (2*PI);
		}
		return(temp);
	}
	
/* vitesse moyenne  appliquer au robot */	
double Vitesse_moyenne(double distance, double Vmoyen, double angle)
	{   
		double Vemoyen;
		
		/* si le robot est trop mal orient vers le point d'arrive */
		if(( angle>THETA0)||(angle<-THETA0))
		{
			Vmoyen = 0;	/* -> le robot tourne sur lui-mme */
		}
		else
		{
			Vemoyen = KDIST * distance;
			/* saturation de la vitesse moyenne de consigne*/
			if (Vemoyen > vitesse_maximum)
			{
				Vemoyen = vitesse_maximum;
			}   
		                                            
			/* limitation de l'acclration de l'volution de la vitesse moyenne */		                                            
			if (abs(((int)(Vemoyen - Vmoyen))) < C)
			{
				Vmoyen = Vemoyen;
			}                    
			else
			{
				if ((Vemoyen - Vmoyen) > 0)
				{
					Vmoyen = Vmoyen + C;
				}                       
				else
				{
					Vmoyen = Vmoyen - C;
				}
			}                           
		}
		
		/****/
		/*Vmoyen = 0;*/
		/****/
		return(Vmoyen);
	}

/* vitesse  appliquer au moteur droit */
double Consigne_droite(double vitesse, double angle)
	{   
		double temp;
		
		if((angle>=0 && angle<THETA0)||(angle<0 && angle>-THETA0))
		{		
		temp = vitesse + KTETA * angle;
		}
		else
		{
		temp = vitesse + gain_rotation * angle;
		}    
		if (temp < -1024)
		{
			temp = -1024;
		}
		if (temp > 1024)
		{
			temp = 1024;
		}
		return(temp);                                                            
	}

/* vitesse  appliquer au moteur gauche */
double Consigne_gauche(double vitesse, double angle)
	{   
		double temp; 
		
		if((angle>=0 && angle<THETA0)||(angle<0 && angle>-THETA0))
		{		
		temp = vitesse - KTETA * angle;
		}
		else
		{
		temp = vitesse - gain_rotation * angle;
		}
		if (temp < -1024)
		{
			temp = -1024;
		}
		if (temp > 1024)
		{
			temp = 1024;
		}
		return(temp);   
	}	

/* cart entre la consigne et la mesure de vitesse */           
double Erreur_vitesse(double consigne, double mesure)
	{   /* 
		if(mesure < -1024)
			mesure = -1024;
		else if(mesure > 1024)
			mesure = 1024;
	    */
	    
		return(consigne - mesure);	
	}

/* fonction  appeler  chaque nouvelle commande de dplacement */
void Initialisation_asservissement(double coord_x_arrivee, double coord_y_arrivee, int deplace, int grotation)
{   
    /* affectation du nouveau point d'arrive */
	x_arrivee = coord_x_arrivee;
	y_arrivee = coord_y_arrivee;
	/* remise  zero des intgrales des lois de rgulation PI des moteurs */
	Integ_Er_droite = 0;
	Integ_Er_gauche = 0;
	/* execution des fonctions d'asservissement */
	ordre_deplacement = ASSERV_RUNNING;                        
	gain_rotation = grotation;
	
	if(deplace == 0)
	{
	  vitesse_maximum = 0;
	} 
	else
	{
	  vitesse_maximum = VMAX;
	}
  
  for(anti_block_pos = 0; anti_block_pos < TEMPS_BLOCAGE; anti_block_pos++)
  {
    x_position_backup[anti_block_pos] = 12.0;
    y_position_backup[anti_block_pos] = 12.0;
    teta_position_backup[anti_block_pos] = 12.0;
  }
  
  anti_block_pos = 0;
}
    
/* regulateur de vitesse droit, fourni la valeur  donner au registre de PWM */    
double Regulateur_PI_droit(double Er)
	{                     
		double Er_corr;
		/* calcul de l'intgrale sur l'erreur */
		Integ_Er_droite = Integ_Er_droite + Er;
		if (Integ_Er_droite > 1024)	Integ_Er_droite = 1024;
		if (Integ_Er_droite < -1024) Integ_Er_droite = -1024;
		/* loi de rgulation PI */
		Er_corr = KP_DROIT * Er + ((KP_DROIT * TE)/TI_DROIT) * Integ_Er_droite;

		/* saturations de la commande en vitesse par la PWM */
		if (Er_corr < -1024) Er_corr = -1024.0;
		if (Er_corr > 1024) Er_corr = 1024.0;
		Er_corr = Er_corr + 1024;	/* offset de l'erreur corrige pour une valeur comprise entre 0 et 2048
			-> valeur  donner au registre de PWM pour la commande de vitesse du moteur droit */                                                                      
		
		/* enregistrement de la commande moteur droite*/                
		if (i< NBPOINTS)
		{   
			if (i==0)
			{
				for(i=0;i<NBPOINTS;i=i+1) tablereg2[i]=0;
				i=0;
			}              
			tablereg2[i]=0;
			tablereg2[i]=(int)(Er_corr);	
		}
		
		
		return(Er_corr);		
	}

/* regulateur de vitesse gauche, fourni la valeur  donner au registre de PWM */    
double Regulateur_PI_gauche(double Er)
	{   
		double Er_corr;
		/* calcul de l'intgrale sur l'erreur */
		Integ_Er_gauche = Integ_Er_gauche + Er;
		if (Integ_Er_gauche > 1024)	Integ_Er_gauche = 1024;
		if (Integ_Er_gauche < -1024) Integ_Er_gauche = -1024;
		/* loi de rgulation */
		Er_corr = KP_GAUCHE * Er + ((KP_GAUCHE * TE)/TI_GAUCHE) * Integ_Er_gauche;

		/* saturations de la commande en vitesse par la PWM */
		if (Er_corr < -1024) Er_corr = -1024.0;
		if (Er_corr > 1024) Er_corr = 1024.0; 
		Er_corr = Er_corr + 1024;	/* offset de l'erreur corrige pour une valeur comprise entre 0 et 2048
			-> valeur  donner au registre de PWM pour la commande de vitesse du moteur gauche */
		
		/* enregistrement de la commande moteur gauche*/                
		if (i< NBPOINTS)
		{   
			if (i==0)
			{
				for(i=0;i<NBPOINTS;i=i+1) tablereg[i]=0;
				i=0;
			}              
			tablereg[i]=0;
			tablereg[i]=(int)(Er_corr);	
		}
		/*
		/*************************************/
		
		return(Er_corr);		
	}           