#include <stdlib.h>
#include <math.h>
#include "x24x_inc.h"
#include "global.h"

extern int ordre_deplacement;  

/* x_robot : position courante en X du robot
 * y_robot : position courante en Y du robot
 */   
extern double x_robot, y_robot,teta_robot;   
extern int timer;

int xbary[1000], ybary[1000], conf[1000], xtmp, ytmp, tmp;

double val;

/* Strategie 1
#define NB_STRATEGIC_POINTS 8
double strategic_points_x[NB_STRATEGIC_POINTS] = { 2.1, 0.3, 0.3, 1.8, 1.8,  0.3, 0.3, 2.1 };
double strategic_points_y[NB_STRATEGIC_POINTS] = { 0.3, 0.3, 0.75, 0.75, 1.35, 1.35, 1.8, 1.8 };
*/

#define NB_STRATEGIC_POINTS 13
double strategic_points_x[NB_STRATEGIC_POINTS] = { 2.1, 1.8, 1.8, 1.5, 1.5, 1.2, 1.2, 0.9, 0.9, 0.6, 0.6, 0.3, 0.3 };
double strategic_points_y[NB_STRATEGIC_POINTS] = { 0.3, 0.3, 1.8, 1.8, 0.3, 0.3, 1.8, 1.8, 0.3, 0.3, 1.8, 1.8, 0.3 };

#define ETAT_BALADE 1
#define ETAT_BALLE_TROUVEE 2
#define ETAT_CHOPPE_BALLE 3
#define ETAT_EVITEMENT_OBSTACLE 4
#define ETAT_MARQUE_BUT 5
#define ETAT_EVITEMENT_MARQUE_BUT 6
#define ETAT_DEPART 7
#define ETAT_FIN 8

void tempo(int n)
{
  int i;
  
  for (i = 0; i < n; i++) { i = i + 1 ; i = i - 1; }
}

int Rotation(double angle, int vitesse)
{          
  if(angle+teta_robot == PI/2)
  {
    Initialisation_asservissement(x_robot, y_robot+1.0, 0, vitesse);    
  }
  else if(angle+teta_robot == -PI/2)
  {
    Initialisation_asservissement(x_robot, y_robot-1.0, 0, vitesse);  
  }
  else if((angle+teta_robot > PI/2) || (angle+teta_robot < -PI/2))  {
    val = tan(PI+angle+teta_robot);
    Initialisation_asservissement(x_robot-1.0, y_robot-val, 0, vitesse);
  }
  else
  {
    val = tan(angle+teta_robot);
    Initialisation_asservissement(x_robot+1.0, y_robot+val, 0, vitesse);
  }    
    
  while(ordre_deplacement == ASSERV_RUNNING);
  
  if(ordre_deplacement == ASSERV_BLOCK)
  {
    return -1;
  }
  
  return 0;
}

int balle_trouvee(void)
{ 
  int ret;
  int barycentre_x, barycentre_y, confiance;
  int i;
  
  
  
  for (i = 0; i < 25; i++) { i = i + 1 ; i = i - 1; }
  
  camera_track_color(RED_COLOR);
  ret = serial_command_wait();
  
  if(ret < 0)
  {
    return 0;
  }
        
  ret = serial_receive_m_command(& barycentre_x, & barycentre_y, & confiance);
  
  if(ret < 0)
  {
    return 0;
  }
  
  if(confiance > 30)
  { 
    return 1;
  }
  
  return 0;
}

/* 0 : succs
 * -1 : balle perdue
 * -2 : robot bloqu
 */
int balle_track(void)
{
  while(1)
  {                                         
      long i;
      int barycentre_x, barycentre_y, confiance;
      int ret;
       
      camera_track_color(RED_COLOR);
      ret = serial_command_wait();
      
      if(ret < 0)
      {
         goto nexttry;
      }
      
      ret = serial_receive_m_command(& barycentre_x, & barycentre_y, & confiance);
      
      if(ret < 0)
      {
         goto nexttry;
      }
      
      if(confiance < 10)
      {
         return -1;
      }
            
      if(barycentre_x < 32)
      {  
         ret = Rotation(PI/16, ROTATION_VITESSE_NORMALE);
         if(ret < 0)
         {
           return -2;
         }
      }
      else if(barycentre_x > 58)
      {
         ret = Rotation(-PI/16, ROTATION_VITESSE_NORMALE);
         if(ret < 0)
         {
           return -2;
         }
      }
      else
      {
         return 0;
      }
      
   nexttry:
      for (i = 0; i < 250; i++) { i = i + 1 ; i = i - 1; }      
    }    


}

/* Fonction qui avance vers la balle en face de nous
 * Retourne 0 si la balle est dans le robot
 * Retourne -1 si la balle est perdue
 * Retourne -2 si blocage dans l'asservissement
 */
int avance_vers_balle(void)
{       
   Initialisation_asservissement(x_robot+cos(teta_robot)*1.0, y_robot+sin(teta_robot)*1.0, 1, ROTATION_VITESSE_NORMALE);

   while(ordre_deplacement == ASSERV_RUNNING)
   {
    long i;
    int barycentre_x, barycentre_y, confiance;
    int ret;
   
    camera_track_color(RED_COLOR);
    ret = serial_command_wait();
      
    if(ret < 0)
    {
      goto nexttry;
    }
      
    ret = serial_receive_m_command(& barycentre_x, & barycentre_y, & confiance);
      
    if(ret < 0)
    {
      goto nexttry;
    }
      
    if(barycentre_x < 32)
    {
      Rotation(PI/16, ROTATION_VITESSE_NORMALE);
    }
    else if(barycentre_x > 58)
    {
      Rotation(-PI/16, ROTATION_VITESSE_NORMALE);
    }
            
    if(barycentre_y < 25)
    {  
      ordre_deplacement = ASSERV_IDLE;
                  
      Initialisation_asservissement(x_robot+cos(teta_robot)*0.35, y_robot+sin(teta_robot)*0.35, 1, ROTATION_VITESSE_NORMALE);
      while(ordre_deplacement == ASSERV_RUNNING);

      if(ordre_deplacement == ASSERV_BLOCK)
      {
        return -2;
      }
                  
      return 0;
    }

    if(confiance < 10)
    { 
      return -1;
    }      
            
  nexttry:
     for (i = 0; i < 250; i++) { i = i + 1 ; i = i - 1; }      
       
   }
  
  if(ordre_deplacement == ASSERV_BLOCK)
  {
    return -2;
  }
  
  return -1;
}


int detect_palmier(void)
{
  int ret;
  int green_barycentre_x, green_barycentre_y, green_confiance;
  int black_barycentre_x, black_barycentre_y, black_confiance;
  int i;
  
  if ((x_robot > 180) && ((teta_robot < ((5*PI)/12)) && (teta_robot > ((-5*PI)/12))))
  {
    return 0;
  }          
  
  if ((x_robot < 60) && ((teta_robot > ((5*PI)/12)) && (teta_robot < ((-5*PI)/12))))
  {
  	return 0;
  }
  
  
  for (i = 0; i < 25; i++) { i = i + 1 ; i = i - 1; }
  
  camera_track_color(GREEN_COLOR);
  ret = serial_command_wait();
  
  if(ret < 0)
  {
    return 0;
  }
        
  ret = serial_receive_m_command(& green_barycentre_x, 
                                 & green_barycentre_y, 
                                 & green_confiance);
  
  if(ret < 0)
  {
    return 0;
  }

  camera_track_color(BLACK_COLOR);
  ret = serial_command_wait();
  
  if(ret < 0)
  {
     return 0;
  }
  
  ret = serial_receive_m_command(& black_barycentre_x, 
                                 & black_barycentre_y, 
                                 & black_confiance);
  
  if(ret < 0)
  {
    return 0;
  }
  
  if((green_confiance > 45) || (black_confiance > 45))
  { 
    return 1;
  }

  return 0;
}

int detect_robot(void)
{ 
  int val;
  
  val = valeur_telemetre();
  
  if(val > 190)
  {
    return 1;
  }
  
  return 0;
}           

/* 0: succs
 * -1: robot bloqu
 */
int marque_but (void)
{ 
  int end;
  int ret;
  
  Initialisation_asservissement(2.15, 1.05, 1, ROTATION_VITESSE_LENTE);
  while (ordre_deplacement == ASSERV_RUNNING);
  
  if(ordre_deplacement == ASSERV_BLOCK)
  {
    return -1;
  }
  
  Initialisation_asservissement(2.30, 1.05, 1, ROTATION_VITESSE_LENTE);
  while (ordre_deplacement == ASSERV_RUNNING);
  
  if(ordre_deplacement == ASSERV_BLOCK)
  {
    return -1;
  }
  
  end = timer - 150;
  
  while(end != timer)
  {
    CMPR1 = 600;
    CMPR2 = 600;
  }             
  
  Rotation(PI, ROTATION_VITESSE_NORMALE);
  
  end = timer - 200;
  
  while(end != timer)
  { 
    CMPR1 = 600;
    CMPR2 = 600;                                                       
  }                                                                    
  
  Initialisation_asservissement(2.15, 1.05, 1, ROTATION_VITESSE_LENTE);
  while (ordre_deplacement == ASSERV_RUNNING);  
    
  if(ordre_deplacement == ASSERV_BLOCK)
  {
    return -1;
  }
  
  return 0;
}

int state;

int states[1000];
int stateidx = 0;  

double distance(double x1, double y1, double x2, double y2)
{
  return sqrt(((y2 - y1) * (y2 - y1)) + ((x2 - x1) * (x2 -x2)));
}       

void debloquage(void)
{
  int end = timer - 30;
  while(timer != end)
  {
    CMPR1 = 400;
    CMPR2 = 400;
  }
  
  Rotation(-PI/3, ROTATION_VITESSE_NORMALE);
  
  Initialisation_asservissement(x_robot + 0.26 * cos(teta_robot), y_robot + 0.26 * sin(teta_robot), 1, ROTATION_VITESSE_NORMALE);
  while(ordre_deplacement == ASSERV_RUNNING);  
}

/*
int action_depart(void)
{
   int end;
   
   Initialisation_asservissement(1.5, 0.0, 1, ROTATION_VITESSE_NORMALE);
   while(ordre_deplacement == ASSERV_RUNNING);
         
   if(ordre_deplacement == ASSERV_BLOCK)
   {
      return -1;
   }                                          
   
   Initialisation_asservissement(2.10, 0.7, 1, ROTATION_VITESSE_LENTE);               
   while(ordre_deplacement == ASSERV_RUNNING);
   
   if(ordre_deplacement == ASSERV_BLOCK)
   {
      return -1;
   }                                          
   
   Initialisation_asservissement (2.30, 0.75, 1, ROTATION_VITESSE_LENTE);
   while(ordre_deplacement == ASSERV_RUNNING);
   
   if(ordre_deplacement == ASSERV_BLOCK)
   {
      return -1;
   }                                          
      
   end = timer - 150;
   
   while(end != timer)
   {           
     CMPR1 = 600;
     CMPR2 = 600;
   }
   
   Rotation(PI, ROTATION_VITESSE_NORMALE);
   
   end = timer - 200;
    
   while(end != timer)
   {
     CMPR1 = 600;
     CMPR2 = 600;
   }
}
*/                     
                                                              
int objectif = 0;
int image = 0;

/* Nouveau systeme de depart */
int nouveau_depart(void)
{
  int end;

  Initialisation_asservissement(0.50, 1.5, 1, ROTATION_VITESSE_NORMALE);
  while(ordre_deplacement == ASSERV_RUNNING);
  
  if(ordre_deplacement == ASSERV_BLOCK)
  {
    return -1;
  }
  
  Initialisation_asservissement(0.9, 1.8, 1, ROTATION_VITESSE_NORMALE);
  while(ordre_deplacement == ASSERV_RUNNING);
  
  if(ordre_deplacement == ASSERV_BLOCK)
  {
    return -1;
  }
  
  Initialisation_asservissement(2.0 , 1.8, 1, ROTATION_VITESSE_NORMALE); 
  while(ordre_deplacement == ASSERV_RUNNING);
  
  if(ordre_deplacement == ASSERV_BLOCK)
  {
    return -1;
  }
  
  Initialisation_asservissement(2.0, 1.05, 1, ROTATION_VITESSE_LENTE);
  while(ordre_deplacement == ASSERV_RUNNING);
  
  if(ordre_deplacement == ASSERV_BLOCK)
  {
    return -1;
  }
  
  Initialisation_asservissement(2.30, 1.05, 1, ROTATION_VITESSE_LENTE);
  while(ordre_deplacement == ASSERV_RUNNING);
  
  if(ordre_deplacement == ASSERV_BLOCK)
  {
    return -1;
  }
  
  end = timer - 150;
   
  while(end != timer)
   {           
     CMPR1 = 600;
     CMPR2 = 600;
   }
   
   Rotation(PI, ROTATION_VITESSE_NORMALE);
   
   end = timer - 200;
    
   while(end != timer)
   {
     CMPR1 = 600;
     CMPR2 = 600;
   }
   
   Initialisation_asservissement(2.0, 1.05, 1, ROTATION_VITESSE_LENTE);
   while(ordre_deplacement == ASSERV_RUNNING);
  
   if(ordre_deplacement == ASSERV_BLOCK)
   {
     return -1;
   }
   
   return 0;
}  

int balle = 0;

void main(void)
{    
	/*booleen pour savoir si il a t dposer la balle au bout de 1mn*/
	int test_minute = 0;
   
    inittimer();
    
    MCRC = MCRC & 0xFFFE;
    PEDATDIR=0x0;    
       
    serial_port_init();
    
    camera_reset();
    serial_command_wait();
 
    camera_pm_mode(1);
    serial_command_wait();
    
    camera_settings();
    serial_command_wait();
    
    for (stateidx = 0; stateidx < 1000; stateidx++)
    {
      states[stateidx] = -1;
    }
    states[0] = 12;
    stateidx = 0;

    while((PEDATDIR & 1) == 0);
    
    timer = 9000;
    
    state = ETAT_DEPART;
    
    x_robot = 0;
    y_robot = 1.5;
    teta_robot = 0;
          
    while(1)
    { 
      if((timer < 3000 ) && (test_minute == 0))
      {
         state = ETAT_MARQUE_BUT;             
         test_minute = 1;         
      }
    
      if(states[stateidx] != state)
      { 
        stateidx++;
        states[stateidx] = state;
      }                                                           
      
      if(state == ETAT_DEPART)
      {
        int ret;
        
        ret = nouveau_depart();
        
        if(ret < 0)
        {
           debloquage();
        }
        
        state = ETAT_BALADE;
      }
      else if(state == ETAT_BALADE)
      {  
        Initialisation_asservissement(strategic_points_x[objectif], strategic_points_y[objectif], 1, ROTATION_VITESSE_NORMALE);
        while(ordre_deplacement == ASSERV_RUNNING)
        {
          if(balle_trouvee() == 1)
          {
            state = ETAT_BALLE_TROUVEE;
            goto nextstate;          
          }        
        }         
        
        /* Pas besoin de grer le blocage : de toute faon, on va changer d'objectif */
        if(ordre_deplacement == ASSERV_BLOCK)
        {
          debloquage();
        }
                                          
        objectif++;
        if(objectif == NB_STRATEGIC_POINTS)
        {
          objectif = 0;
        }           
      }
      else if(state == ETAT_BALLE_TROUVEE)
      {  
         int tst;
         
         ordre_deplacement = ASSERV_IDLE;
         tst = balle_track();
         if(tst == -2)
         {
           debloquage();
           state = ETAT_BALADE;
         }
         if(tst == -1)
         { 
           /* Balle perdue => on repart en balade */
           state = ETAT_BALADE;
         }
         else
         {
           state = ETAT_CHOPPE_BALLE;
         }
      }
      else if(state == ETAT_CHOPPE_BALLE)
      {
         int tst;
         
         tst = avance_vers_balle();
         if(tst == -1)
         {
            /* Balle perdue => on repart en balade */
            state = ETAT_BALADE;
         }
         else if(tst == -2)
         {
           /* Blocage asservissement */
           debloquage();
           state = ETAT_BALADE;
         }
         else
         {
            state = ETAT_MARQUE_BUT;
         }
      }
      else if(state == ETAT_MARQUE_BUT)
      {
         int ret;
         
         ret = marque_but();         
         
         if(ret < 0)
         {
           debloquage();
           state = ETAT_MARQUE_BUT;
         }
         else
         {
           state = ETAT_BALADE;
         }
      }
      else
      {
        break;
      }
      nextstate:
        continue;    
    }
    
  while(1);
}